@@ -3,14 +3,15 @@ import json
 import logging
 import re
 import traceback
-from typing import Dict, Optional
+from typing import Callable, Dict, List, Optional, Set
 import urllib.parse
 
-from apispec import yaml_utils
+from apispec import APISpec, yaml_utils
+from apispec.exceptions import DuplicateComponentNameError
 from flask import Blueprint, current_app, jsonify, make_response, request, Response
 from flask_babel import lazy_gettext as _
 import jsonschema
-from marshmallow import ValidationError
+from marshmallow import Schema, ValidationError
 from marshmallow_sqlalchemy.fields import Related, RelatedList
 import prison
 from sqlalchemy.exc import IntegrityError
@@ -57,6 +58,7 @@ from ..const import (
     PERMISSION_PREFIX,
 )
 from ..exceptions import FABException, InvalidOrderByColumnFABException
+from ..hooks import get_before_request_hooks, wrap_route_handler_with_hooks
 from ..security.decorators import permission_name, protect
 
 log = logging.getLogger(__name__)
@@ -213,22 +215,22 @@ class BaseApi(object):
 
     appbuilder = None
     blueprint = None
-    endpoint = None
+    endpoint: Optional[str] = None
 
-    version = "v1"
+    version: Optional[str] = "v1"
     """
         Define the Api version for this resource/class
     """
-    route_base = None
+    route_base: Optional[str] = None
     """
         Define the route base where all methods will suffix from
     """
-    resource_name = None
+    resource_name: Optional[str] = None
     """
         Defines a custom resource name, overrides the inferred from Class name
         makes no sense to use it with route base
     """
-    base_permissions = None
+    base_permissions: Optional[List[str]] = None
     """
         A list of allowed base permissions::
 
@@ -236,16 +238,16 @@ class BaseApi(object):
                 base_permissions = ['can_get']
 
     """
-    class_permission_name = None
+    class_permission_name: Optional[str] = None
     """
         Override class permission name default fallback to self.__class__.__name__
     """
-    previous_class_permission_name = None
+    previous_class_permission_name: Optional[str] = None
     """
         If set security converge will replace all permissions tuples
         with this name by the class_permission_name or self.__class__.__name__
     """
-    method_permission_name = None
+    method_permission_name: Optional[Dict[str, str]] = None
     """
         Override method permission names, example::
 
@@ -257,7 +259,7 @@ class BaseApi(object):
                 'delete': 'write'
             }
     """
-    previous_method_permission_name = None
+    previous_method_permission_name: Optional[Dict[str, str]] = None
     """
         Use same structure as method_permission_name. If set security converge
         will replace all method permissions by the new ones
@@ -271,7 +273,7 @@ class BaseApi(object):
     """
         If using flask-wtf CSRFProtect exempt the API from check
     """
-    apispec_parameter_schemas = None
+    apispec_parameter_schemas: Optional[Dict[str, Dict]] = None
     """
         Set your custom Rison parameter schemas here so that
         they get registered on the OpenApi spec::
@@ -376,7 +378,7 @@ class BaseApi(object):
 
         The previous examples will only register the `put`, `post` and `delete` routes
     """
-    include_route_methods = None
+    include_route_methods: Set[str] = None
     """
         If defined will assume a white list setup, where all endpoints are excluded
         except those define on this attribute
@@ -411,7 +413,7 @@ class BaseApi(object):
         Use this attribute to override the tag name
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         """
             Initialization of base permissions
             based on exposed methods and actions
@@ -484,7 +486,8 @@ class BaseApi(object):
         self._register_urls()
         return self.blueprint
 
-    def add_api_spec(self, api_spec):
+    def add_api_spec(self, api_spec: APISpec) -> None:
+        self.add_apispec_components(api_spec)
         for attr_name in dir(self):
             attr = getattr(self, attr_name)
             if hasattr(attr, "_urls"):
@@ -509,27 +512,18 @@ class BaseApi(object):
                             self.openapi_spec_tag or self.__class__.__name__
                         )
                         api_spec._paths[path][operation]["tags"] = [openapi_spec_tag]
-        self.add_apispec_components(api_spec)
 
-    def add_apispec_components(self, api_spec):
+    def add_apispec_components(self, api_spec: APISpec) -> None:
         for k, v in self.responses.items():
             api_spec.components._responses[k] = v
         for k, v in self._apispec_parameter_schemas.items():
-            if k not in api_spec.components._parameters:
-                _v = {
-                    "in": "query",
-                    "name": API_URI_RIS_KEY,
-                    "content": {
-                        "application/json": {
-                            "schema": {"$ref": "#/components/schemas/{}".format(k)}
-                        }
-                    },
-                }
-                # Using private because parameter method does not behave correctly
-                api_spec.components._schemas[k] = v
-                api_spec.components._parameters[k] = _v
+            try:
+                api_spec.components.schema(k, v)
+            except DuplicateComponentNameError:
+                pass
 
-    def _register_urls(self):
+    def _register_urls(self) -> None:
+        before_request_hooks = get_before_request_hooks(self)
         for attr_name in dir(self):
             if (
                 self.include_route_methods is not None
@@ -545,11 +539,18 @@ class BaseApi(object):
                     log.info(
                         f"Registering route {self.blueprint.url_prefix}{url} {methods}"
                     )
-                    self.blueprint.add_url_rule(url, attr_name, attr, methods=methods)
+                    route_handler = wrap_route_handler_with_hooks(
+                        attr_name, attr, before_request_hooks
+                    )
+                    self.blueprint.add_url_rule(
+                        url, attr_name, route_handler, methods=methods
+                    )
 
-    def path_helper(self, path=None, operations=None, **kwargs):
+    def path_helper(
+        self, path: str = None, operations: Dict[str, Dict] = None, **kwargs
+    ) -> str:
         """
-            Works like a apispec plugin
+            Works like an apispec plugin
             May return a path as string and mutate operations dict.
 
         :param str path: Path to the resource
@@ -561,7 +562,7 @@ class BaseApi(object):
         """
         RE_URL = re.compile(r"<(?:[^:<>]+:)?([^<>]+)>")
         path = RE_URL.sub(r"{\1}", path)
-        return "/{}{}".format(self.resource_name, path)
+        return f"/{self.resource_name}{path}"
 
     def operation_helper(
         self, path=None, operations=None, methods=None, func=None, **kwargs
@@ -861,72 +862,83 @@ class ModelRestApi(BaseModelApi):
         List Title, if not configured the default is
         'List ' with pretty model name
     """
-    show_title = ""
+    show_title: Optional[str] = ""
     """
         Show Title , if not configured the default is
         'Show ' with pretty model name
     """
-    add_title = ""
+    add_title: Optional[str] = ""
     """
         Add Title , if not configured the default is
         'Add ' with pretty model name
     """
-    edit_title = ""
+    edit_title: Optional[str] = ""
     """
         Edit Title , if not configured the default is
         'Edit ' with pretty model name
     """
-
-    list_columns = None
+    list_select_columns: Optional[List[str]] = None
+    """
+        A List of column names that will be included on the SQL select.
+        This is useful for including all necessary columns that are referenced
+        by properties listed on `list_columns` without generating N+1 queries.
+    """
+    list_columns: Optional[List[str]] = None
     """
         A list of columns (or model's methods) to be displayed on the list view.
         Use it to control the order of the display
     """
-    show_columns = None
+    show_select_columns: Optional[List[str]] = None
+    """
+        A List of column names that will be included on the SQL select.
+        This is useful for including all necessary columns that are referenced
+        by properties listed on `show_columns` without generating N+1 queries.
+    """
+    show_columns: Optional[List[str]] = None
     """
         A list of columns (or model's methods) for the get item endpoint.
         Use it to control the order of the results
     """
-    add_columns = None
+    add_columns: Optional[List[str]] = None
     """
         A list of columns (or model's methods) to be allowed to post
     """
-    edit_columns = None
+    edit_columns: Optional[List[str]] = None
     """
         A list of columns (or model's methods) to be allowed to update
     """
-    list_exclude_columns = None
+    list_exclude_columns: Optional[List[str]] = None
     """
         A list of columns to exclude from the get list endpoint.
         By default all columns are included.
     """
-    show_exclude_columns = None
+    show_exclude_columns: Optional[List[str]] = None
     """
         A list of columns to exclude from the get item endpoint.
         By default all columns are included.
     """
-    add_exclude_columns = None
+    add_exclude_columns: Optional[List[str]] = None
     """
         A list of columns to exclude from the add endpoint.
         By default all columns are included.
     """
-    edit_exclude_columns = None
+    edit_exclude_columns: Optional[List[str]] = None
     """
         A list of columns to exclude from the edit endpoint.
         By default all columns are included.
     """
-    order_columns = None
+    order_columns: Optional[List[str]] = None
     """ Allowed order columns """
     page_size = 20
     """
         Use this property to change default page size
     """
-    max_page_size = None
+    max_page_size: Optional[int] = None
     """
         class override for the FAB_API_MAX_SIZE, use special -1 to allow for any page
         size
     """
-    description_columns = None
+    description_columns: Optional[Dict[str, str]] = None
     """
         Dictionary with column descriptions that will be shown on the forms::
 
@@ -936,8 +948,8 @@ class ModelRestApi(BaseModelApi):
                 description_columns = {'name':'your models name column',
                                         'address':'the address column'}
     """
-    validators_columns = None
-    """ Dictionary to add your own validators for forms """
+    validators_columns: Optional[Dict[str, Callable]] = None
+    """ Dictionary to add your own marshmallow validators """
 
     add_query_rel_fields = None
     """
@@ -979,22 +991,22 @@ class ModelRestApi(BaseModelApi):
                     'gender': ('name', 'asc')
                 }
     """
-    list_model_schema = None
+    list_model_schema: Optional[Schema] = None
     """
         Override to provide your own marshmallow Schema
         for JSON to SQLA dumps
     """
-    add_model_schema = None
+    add_model_schema: Optional[Schema] = None
     """
         Override to provide your own marshmallow Schema
         for JSON to SQLA dumps
     """
-    edit_model_schema = None
+    edit_model_schema: Optional[Schema] = None
     """
         Override to provide your own marshmallow Schema
         for JSON to SQLA dumps
     """
-    show_model_schema = None
+    show_model_schema: Optional[Schema] = None
     """
         Override to provide your own marshmallow Schema
         for JSON to SQLA dumps
@@ -1021,42 +1033,60 @@ class ModelRestApi(BaseModelApi):
         self._init_model_schemas()
         return super(ModelRestApi, self).create_blueprint(appbuilder, *args, **kwargs)
 
+    @property
+    def list_model_schema_name(self) -> str:
+        return f"{self.__class__.__name__}.get_list"
+
+    @property
+    def show_model_schema_name(self) -> str:
+        return f"{self.__class__.__name__}.get"
+
+    @property
+    def add_model_schema_name(self) -> str:
+        return f"{self.__class__.__name__}.post"
+
+    @property
+    def edit_model_schema_name(self) -> str:
+        return f"{self.__class__.__name__}.put"
+
     def add_apispec_components(self, api_spec):
         super(ModelRestApi, self).add_apispec_components(api_spec)
         api_spec.components.schema(
-            "{}.{}".format(self.__class__.__name__, "get_list"),
-            schema=self.list_model_schema,
+            self.list_model_schema_name, schema=self.list_model_schema
         )
         api_spec.components.schema(
-            "{}.{}".format(self.__class__.__name__, "post"),
-            schema=self.add_model_schema,
+            self.add_model_schema_name, schema=self.add_model_schema
         )
         api_spec.components.schema(
-            "{}.{}".format(self.__class__.__name__, "put"),
-            schema=self.edit_model_schema,
+            self.edit_model_schema_name, schema=self.edit_model_schema
         )
         api_spec.components.schema(
-            "{}.{}".format(self.__class__.__name__, "get"),
-            schema=self.show_model_schema,
+            self.show_model_schema_name, schema=self.show_model_schema
         )
 
     def _init_model_schemas(self):
         # Create Marshmalow schemas if one is not specified
         if self.list_model_schema is None:
             self.list_model_schema = self.model2schemaconverter.convert(
-                self.list_columns
+                self.list_columns, parent_schema_name=self.list_model_schema_name
             )
         if self.add_model_schema is None:
             self.add_model_schema = self.model2schemaconverter.convert(
-                self.add_columns, nested=False, enum_dump_by_name=True
+                self.add_columns,
+                nested=False,
+                enum_dump_by_name=True,
+                parent_schema_name=self.add_model_schema_name,
             )
         if self.edit_model_schema is None:
             self.edit_model_schema = self.model2schemaconverter.convert(
-                self.edit_columns, nested=False, enum_dump_by_name=True
+                self.edit_columns,
+                nested=False,
+                enum_dump_by_name=True,
+                parent_schema_name=self.edit_model_schema_name,
             )
         if self.show_model_schema is None:
             self.show_model_schema = self.model2schemaconverter.convert(
-                self.show_columns
+                self.show_columns, parent_schema_name=self.show_model_schema_name
             )
 
     def _init_titles(self):
@@ -1075,7 +1105,7 @@ class ModelRestApi(BaseModelApi):
             self.show_title = "Show " + self._prettify_name(class_name)
         self.title = self.list_title
 
-    def _init_properties(self):
+    def _init_properties(self) -> None:
         """
             Init Properties
         """
@@ -1097,6 +1127,7 @@ class ModelRestApi(BaseModelApi):
                 for x in self.datamodel.get_user_columns_list()
                 if x not in self.list_exclude_columns
             ]
+        self.list_select_columns = self.list_select_columns or self.list_columns
 
         self.order_columns = (
             self.order_columns
@@ -1107,6 +1138,8 @@ class ModelRestApi(BaseModelApi):
             self.show_columns = [
                 x for x in list_cols if x not in self.show_exclude_columns
             ]
+        self.show_select_columns = self.show_select_columns or self.show_columns
+
         if not self.add_columns:
             self.add_columns = [
                 x for x in list_cols if x not in self.add_exclude_columns
@@ -1247,8 +1280,15 @@ class ModelRestApi(BaseModelApi):
         """ Endpoint that renders a response for CRUD REST meta data
         ---
         get:
+          description: >-
+            Get metadata information about this API resource
           parameters:
-          - $ref: '#/components/parameters/get_info_schema'
+          - in: query
+            name: q
+            content:
+              application/json:
+                schema:
+                  $ref: '#/components/schemas/get_info_schema'
           responses:
             200:
               description: Item from Model
@@ -1263,7 +1303,22 @@ class ModelRestApi(BaseModelApi):
                         type: object
                       filters:
                         type: object
+                        properties:
+                          column_name:
+                            type: array
+                            items:
+                              type: object
+                              properties:
+                                name:
+                                  description: >-
+                                    The filter name. Will be translated by babel
+                                  type: string
+                                operator:
+                                  description: >-
+                                    The filter operation key to use on list filters
+                                  type: string
                       permissions:
+                        description: The user permissions for this API resource
                         type: array
                         items:
                           type: string
@@ -1286,7 +1341,7 @@ class ModelRestApi(BaseModelApi):
         :param kwargs: Query string parameter arguments
         :return: HTTP Response
         """
-        item = self.datamodel.get(pk, self._base_filters)
+        item = self.datamodel.get(pk, self._base_filters, self.show_select_columns)
         if not item:
             return self.response_404()
 
@@ -1306,7 +1361,7 @@ class ModelRestApi(BaseModelApi):
             _show_model_schema = self.show_model_schema
 
         _response["id"] = pk
-        _response[API_RESULT_RES_KEY] = _show_model_schema.dump(item, many=False).data
+        _response[API_RESULT_RES_KEY] = _show_model_schema.dump(item, many=False)
         self.pre_get(_response)
         return self.response(200, **_response)
 
@@ -1323,12 +1378,19 @@ class ModelRestApi(BaseModelApi):
         """Get item from Model
         ---
         get:
+          description: >-
+            Get an item model
           parameters:
           - in: path
             schema:
               type: integer
             name: pk
-          - $ref: '#/components/parameters/get_item_schema'
+          - in: query
+            name: q
+            content:
+              application/json:
+                schema:
+                  $ref: '#/components/schemas/get_item_schema'
           responses:
             200:
               description: Item from Model
@@ -1339,15 +1401,36 @@ class ModelRestApi(BaseModelApi):
                     properties:
                       label_columns:
                         type: object
+                        properties:
+                          column_name:
+                            description: >-
+                              The label for the column name.
+                              Will be translated by babel
+                            example: A Nice label for the column
+                            type: string
                       show_columns:
+                        description: >-
+                          A list of columns
                         type: array
                         items:
                           type: string
                       description_columns:
                         type: object
+                        properties:
+                          column_name:
+                            description: >-
+                              The description for the column name.
+                              Will be translated by babel
+                            example: A Nice description for the column
+                            type: string
                       show_title:
+                        description: >-
+                          A title to render.
+                          Will be translated by babel
+                        example: Show Item Details
                         type: string
                       id:
+                        description: The item id
                         type: string
                       result:
                         $ref: '#/components/schemas/{{self.__class__.__name__}}.get'
@@ -1373,13 +1456,15 @@ class ModelRestApi(BaseModelApi):
         # handle select columns
         select_cols = _args.get(API_SELECT_COLUMNS_RIS_KEY, [])
         _pruned_select_cols = [col for col in select_cols if col in self.list_columns]
+        # map decorated metadata
         self.set_response_key_mappings(
             _response,
             self.get_list,
             _args,
             **{API_SELECT_COLUMNS_RIS_KEY: _pruned_select_cols},
         )
-
+        # Create a response schema with the computed response columns,
+        # defined or requested
         if _pruned_select_cols:
             _list_model_schema = self.model2schemaconverter.convert(_pruned_select_cols)
         else:
@@ -1397,17 +1482,16 @@ class ModelRestApi(BaseModelApi):
         # handle pagination
         page_index, page_size = self._handle_page_args(_args)
         # Make the query
-        query_select_columns = _pruned_select_cols or self.list_columns
         count, lst = self.datamodel.query(
             joined_filters,
             order_column,
             order_direction,
             page=page_index,
             page_size=page_size,
-            select_columns=query_select_columns,
+            select_columns=self.list_select_columns,
         )
         pks = self.datamodel.get_keys(lst)
-        _response[API_RESULT_RES_KEY] = _list_model_schema.dump(lst, many=True).data
+        _response[API_RESULT_RES_KEY] = _list_model_schema.dump(lst, many=True)
         _response["ids"] = pks
         _response["count"] = count
         self.pre_get_list(_response)
@@ -1427,8 +1511,15 @@ class ModelRestApi(BaseModelApi):
         """Get list of items from Model
         ---
         get:
+          description: >-
+            Get a list of models
           parameters:
-          - $ref: '#/components/parameters/get_list_schema'
+          - in: query
+            name: q
+            content:
+              application/json:
+                schema:
+                  $ref: '#/components/schemas/get_list_schema'
           responses:
             200:
               description: Items from Model
@@ -1439,28 +1530,56 @@ class ModelRestApi(BaseModelApi):
                     properties:
                       label_columns:
                         type: object
+                        properties:
+                          column_name:
+                            description: >-
+                              The label for the column name.
+                              Will be translated by babel
+                            example: A Nice label for the column
+                            type: string
                       list_columns:
+                        description: >-
+                          A list of columns
                         type: array
                         items:
                           type: string
                       description_columns:
                         type: object
+                        properties:
+                          column_name:
+                            description: >-
+                              The description for the column name.
+                              Will be translated by babel
+                            example: A Nice description for the column
+                            type: string
                       list_title:
+                        description: >-
+                          A title to render.
+                          Will be translated by babel
+                        example: List Items
                         type: string
                       ids:
+                        description: >-
+                          A list of item ids, useful when you don't know the column id
                         type: array
                         items:
                           type: string
                       count:
+                        description: >-
+                          The total record count on the backend
                         type: number
                       order_columns:
+                        description: >-
+                          A list of allowed columns to sort
                         type: array
                         items:
                           type: string
                       result:
-                          type: array
-                          items:
-                            $ref: '#/components/schemas/{{self.__class__.__name__}}.get_list'  # noqa
+                        description: >-
+                          The result from the get list query
+                        type: array
+                        items:
+                          $ref: '#/components/schemas/{{self.__class__.__name__}}.get_list'  # noqa
             400:
               $ref: '#/components/responses/400'
             401:
@@ -1484,19 +1603,15 @@ class ModelRestApi(BaseModelApi):
         except ValidationError as err:
             return self.response_422(message=err.messages)
         # This validates custom Schema with custom validations
-        if isinstance(item.data, dict):
-            return self.response_422(message=item.errors)
-        self.pre_add(item.data)
+        self.pre_add(item)
         try:
-            self.datamodel.add(item.data, raise_exception=True)
-            self.post_add(item.data)
+            self.datamodel.add(item, raise_exception=True)
+            self.post_add(item)
             return self.response(
                 201,
                 **{
-                    API_RESULT_RES_KEY: self.add_model_schema.dump(
-                        item.data, many=False
-                    ).data,
-                    "id": self.datamodel.get_pk_value(item.data),
+                    API_RESULT_RES_KEY: self.add_model_schema.dump(item, many=False),
+                    "id": self.datamodel.get_pk_value(item),
                 },
             )
         except IntegrityError as e:
@@ -1554,20 +1669,13 @@ class ModelRestApi(BaseModelApi):
             item = self.edit_model_schema.load(data, instance=item)
         except ValidationError as err:
             return self.response_422(message=err.messages)
-        # This validates custom Schema with custom validations
-        if isinstance(item.data, dict):
-            return self.response_422(message=item.errors)
-        self.pre_update(item.data)
+        self.pre_update(item)
         try:
-            self.datamodel.edit(item.data, raise_exception=True)
+            self.datamodel.edit(item, raise_exception=True)
             self.post_update(item)
             return self.response(
                 200,
-                **{
-                    API_RESULT_RES_KEY: self.edit_model_schema.dump(
-                        item.data, many=False
-                    ).data
-                },
+                **{API_RESULT_RES_KEY: self.edit_model_schema.dump(item, many=False)},
             )
         except IntegrityError as e:
             return self.response_422(message=str(e.orig))
@@ -1828,7 +1936,7 @@ class ModelRestApi(BaseModelApi):
         :param data: python data structure
         :return: python data structure
         """
-        data_item = self.edit_model_schema.dump(model_item, many=False).data
+        data_item = self.edit_model_schema.dump(model_item, many=False)
         for _col in self.edit_columns:
             if _col not in data.keys():
                 data[_col] = data_item[_col]
